@@ -25,8 +25,6 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include <Python.h>
-#include <string.h>
 #include "smbcmodule.h"
 #include "context.h"
 #include "dir.h"
@@ -143,16 +141,6 @@ Context_init (Context *self, PyObject *args, PyObject *kwds)
       self->auth_fn = auth;
     }
 
-#if SMBCLIENT_VERSION < 500 /* Older than 0.5.0 */
-  if(proto)
-  {
-    debugprintf ("-> Setting client max protocol to %s\n", proto);
-    lp_set_cmdline("client max protocol", proto);
-    debugprintf ("-> Setting client min protocol to %s\n", proto);
-    lp_set_cmdline("client min protocol", proto);
-  }
-#endif
-
   debugprintf ("-> Context_init ()\n");
 
   errno = 0;
@@ -170,13 +158,17 @@ Context_init (Context *self, PyObject *args, PyObject *kwds)
   smbc_setOptionUserData (ctx, self);
   if (auth)
     smbc_setFunctionAuthDataWithContext (ctx, auth_fn);
-#if SMBCLIENT_VERSION >= 500 /* 0.5.0 or newer */
   if(proto)
   {
-    debugprintf ("-> Setting client min/max protocol to %s\n", proto);
-    smbc_setOptionProtocols (ctx, proto, proto);
-  }
+#if SMBCLIENT_VERSION >= 500 /* 0.5.0 or newer */
+    debugprintf("-> Setting client min/max protocol to %s by smbc_setOptionProtocols\n", proto);
+    smbc_setOptionProtocols(ctx, proto, proto);
+#else
+    debugprintf("-> Setting client min/max protocol to %s by smbc_option_set\n", proto);
+    smbc_option_set(ctx, "client max protocol", proto);
+    smbc_option_set(ctx, "client min protocol", proto);
 #endif
+  }
 
   if (smbc_init_context (ctx) == NULL)
     {
@@ -586,64 +578,43 @@ Context_chmod (Context *self, PyObject *args)
  */
 static PyObject *
 Context_getxattr (Context *self, PyObject *args)
-{
-  int ret;
-  char *uri = NULL;
-  char *name = NULL;
-  char *buffer = NULL;
-  static smbc_getxattr_fn fn;
-
-  // smbc_getxattr takes two string parameters
-  if (!PyArg_ParseTuple (args, "ss", &uri, &name))
-    {
-      return NULL;
-    }
-
-  /* The security descriptor string returned by this call will vary depending on the requested attribute
-   * A call with system.nt_sec_desc.* will return the longest string which would be in the following format:
-   *
-   * REVISION:<revision number>,OWNER:<sid>,GROUP:<sid>,ACL:<sid>:<type>/<flags>/<mask>
-   *
-   * There could be multiple ACL entries up to a reasonable maximum of 1820.
-   *
-   * <revision number> : 3 chars
-   * <sid> :  184 chars
-   * <type>:  1 char
-   * <flags>: 3 chars
-   * <mask>:  10 chars
-   *
-   * The maximum size of the security descriptor string returned can be
-   * derived as follows (includes space for terminating null):
-   * Sec Desc = 13 + 2 x (7 + <sid>) + 1820 * (5 + <acl>) = 375315
-   *
-   * References: https://msdn.microsoft.com/en-us/library/cc246018.aspx
-   *             https://technet.microsoft.com/en-us/library/cc961995.aspx
-   *             https://technet.microsoft.com/en-us/library/cc961986.aspx
-   */
-
-  size_t size = 375315;
-  buffer = (char *)malloc (size);
-  if(!buffer)
-    return PyErr_NoMemory ();
-
-  bzero(buffer, size);
-
-  errno = 0;
-  fn = smbc_getFunctionGetxattr(self->context);
-  ret = (*fn)(self->context, uri, name, buffer, size);
-
-  if (ret < 0)
-    {
-      pysmbc_SetFromErrno ();
-      free(buffer);
-      return NULL;
-    }
-
-  PyObject *value = PyUnicode_FromString(buffer);
-  free(buffer);
-
-  return value;
-}
+  {
+    PyObject * result = NULL;
+    char *uri = NULL;
+    char *name = NULL;
+    char *buffer = NULL;
+    int ret;
+    do /*once*/
+      {
+        if (!PyArg_ParseTuple(args, "ss", &uri, &name))
+            break;
+        const smbc_getxattr_fn fn = smbc_getFunctionGetxattr(self->context);
+        errno = 0;
+        ret = fn(self->context, uri, name, NULL, 0);
+        if (ret < 0)
+          {
+            pysmbc_SetFromErrno();
+            break;
+          }
+	const int bufsize = ret + 1;
+        buffer = (char *)malloc(bufsize);
+        if (buffer == NULL)
+          {
+            PyErr_NoMemory();
+            break;
+          } /*if*/
+        ret = fn(self->context, uri, name, buffer, bufsize);
+        if (ret < 0)
+          {
+            pysmbc_SetFromErrno();
+            break;
+          } /*if*/
+        result = PyUnicode_FromString(buffer);
+      }
+    while (false);
+    free(buffer);
+    return result;
+  } /*Context_getxattr*/
 
 
 /**
@@ -805,7 +776,12 @@ Context_setNetbiosName (Context *self, PyObject *value, void *closure)
       return -1;
     }
 
-  chars = PyUnicode_GetSize (value); /* not including NUL */
+#if PY_MAJOR_VERSION > 3
+  chars = PyUnicode_GET_LENGTH(value);
+#else
+  chars = PyUnicode_GET_SIZE(value); /* not including NUL */
+#endif
+
   w_name = malloc ((chars + 1) * sizeof (wchar_t));
   if (!w_name)
     {
@@ -874,7 +850,12 @@ Context_setWorkgroup (Context *self, PyObject *value, void *closure)
       return -1;
     }
 
-  chars = PyUnicode_GetSize (value); /* not including NUL */
+#if PY_MAJOR_VERSION > 3
+  chars = PyUnicode_GET_LENGTH(value);
+#else
+  chars = PyUnicode_GET_SIZE(value); /* not including NUL */
+#endif
+
   w_workgroup = malloc ((chars + 1) * sizeof (wchar_t));
   if (!w_workgroup)
     {
